"""OS routines for Plan 9 from Bell Labs

System calls are implemented by the _plan9 module.
Various wrappers are provided here.

"""

from plan9 import *
from plan9 import _exits
import string

MORDER	= 0x0003		# mask for bits defining order of mounting
MREPL	= 0x0000		# mount replaces object
MBEFORE	= 0x0001		# mount goes before others in union
MAFTER	= 0x0002		# mount goes after others in union
MCREATE	= 0x0004		# permit creation in mounted directory
MCACHE	= 0x0010		# cache some data
MMASK	= 0x0007		# all bits on

OREAD	= 0			# open for read
OWRITE	= 1			# write
ORDWR	= 2			# read and write
OEXEC	= 3			# execute, == read but check execute permission
OTRUNC	= 16			# or'ed in (except for exec), truncate file first
OCEXEC	= 32			# or'ed in, close on exec
ORCLOSE	= 64			# or'ed in, remove on close
OEXCL	= 0x1000		# or'ed in, exclusive use (create only)

AEXIST	= 0			# accessible: exists 
AEXEC	= 1			# execute access
AWRITE	= 2			# write access
AREAD	= 4			# read access

QTDIR	= 0x80		# type bit for directories
QTAPPEND = 0x40		# type bit for append only files
QTEXCL	= 0x20		# type bit for exclusive use files
QTMOUNT = 0x10		# type bit for mounted channel
QTAUTH	= 0x08		# type bit for authentication file
QTFILE	= 0x00		# plan file

DMDIR	= 0x80000000	# mode bit for directories
DMAPPEND = 0x40000000 # mode bit for append only files
DMEXCL	= 0x20000000	# mode bit for exclusive use files
DMMOUNT = 0x10000000 # mode bit for mounted channel
DMAUTH	= 0x08000000	# mode bit for authentication file
DMREAD	= 0x4		# mode bit for read permission
DMWRITE	= 0x2		# mode bit for write permission
DMEXEC	= 0x1		# mode bit for execute permission

RFNAMEG	= 1<<0		# new name space group
RFENVG	= 1<<1		# new environment group
RFFDG	= 1<<2		# new file descriptor group
RFNOTEG	= 1<<3		# new note group
RFPROC	= 1<<4		# new process
RFMEM	= 1<<5		# share memory
RFNOWAIT = 1<<6		# don't generate wait message for new process
RFCNAMEG = 1<<10		# clear name space
RFCENVG	= 1<<11		# clear environment
RFCFDG	= 1<<12		# clear file descriptors
RFREND	= 1<<13		# new rendezvous group
RFNOMNT = 1<<14		# don't allow child to mount

class Qid:
	def __init__(self):
		self.path1 = 0L
		self.vers = 0
		self.type = 0

	def fromtuple(self, x):
		self.path = x[0]
		self.vers = x[1]
		self.type = x[2]
		return self

	def totuple(self):
		return (self.path, self.vers, self.type)

	def __repr__(self):
		return '<Qid: 0x%lx, %d, 0x%x>' % (self.path, self.vers, self.type)

class Dir:
	def __init__(self):
		self.type = 0xFFFFFFFF
		self.dev = '\x80'
		self.qid = Qid().fromtuple((0xFFFFFFFFFFFFFFFFL, 0xFFFFFFFF, 0xFF))
		self.mode = 0xFFFFFFFF
		self.atime = 0xFFFFFFFF
		self.mtime = 0xFFFFFFFF
		self.length = 0xFFFFFFFFFFFFFFFFL
		self.name = ''
		self.uid = ''
		self.gid = ''
		self.muid = ''

	def fromtuple(self, x):
		d = Dir()
		d.name = x[0]
		d.qid = Qid().fromtuple(x[1])
		d.dev = x[2][0]
		d.type = x[2][1]
		d.mode = x[3]
		d.atime = x[4]
		d.mtime = x[5]
		d.length = x[6]
		d.uid = x[7]
		d.gid = x[8]
		d.muid = x[9]
		return d

	def totuple(self):
		return (self.name, self.qid, (self.dev, self.type), self.mode, self.atime, \
			self.mtime, self.length, self.uid, self.gid, self.muid)

	def __repr__(self):
		return '<Dir: %s %s #%s%d %lo %ld %ld %ld %s %s %s>' % \
			(self.name, str(self.qid), self.dev, self.type, self.mode, self.atime, \
			self.mtime, self.length, self.uid, self.gid, self.muid)

class Waitmsg:
	def __init__(self):
		self.pid = -1
		self.utime = 0
		self.stime = 0
		self.rtime = 0
		self.msg = ''

	def fromtuple(self, x):
		self.pid = x[0]
		self.utime = x[1]
		self.stime = x[2]
		self.rtime = x[3]
		self.msg = x[4]
		return self

	def totuple(self):
		return (self.pid, self.utime, self.stime, self.rtime, self.msg)

	def __repr__(self):
		return '<Waitmsg: %d %ld %ld %ld %s>' % self.totuple()

def fork():
	return rfork(RFFDG|RFREND|RFPROC)

def isatty(fd):
	s = fd2path(fd);
	return len(s)>=9 and s[-9:]=='/dev/cons'

def dirstat(path):
	return Dir().fromtuple(stat(path))

def dirwstat(path, dir):
	return wstat(path, dir.totuple())

def dirfstat(fd):
	return Dir().fromtuple(fstat(fd))

def dirfwstat(fd, dir):
	return fwstat(fd, dir.totuple())

def getpid():
	return string.atoi(readfile("/dev/pid"))

def getppid():
	return string.atoi(readfile("/dev/ppid"))

def getuser():
	return readfile("/dev/user")

def getwd():
	try:
		fd = -1
		fd = open(".", OREAD)
		s = fd2path(fd)
		close(fd)
		return s
	except:
		close(fd)
		raise

def read(fd, len):
	return pread(fd, len, -1)

def readfile(path):
	fd = open(path, OREAD)
	tot = ''
	try:
		s = read(fd)
		while s != '':
			tot += s
			s = read(fd)
	except:
		close(fd)
		raise
	close(fd)
	return tot

def readn(fd, m):
	tot = ''
	s = read(fd, m)
	while s != '':
		tot += s
		s = read(fd, m - len(tot))
	return tot
			
def sysname():
	return readfile("/env/sysname")

def time():
	return readfile("/dev/time")

def write(fd, s):
	return pwrite(fd, s, -1)

def wait():
	return Waitmsg().fromtuple(tokenize(await()))

def waitpid():
	return string.atoi(tokenize(await())[0])

